/*
 * Copyright 2014 Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License or (at your option) version 3 or any later version
 * accepted by the membership of KDE e.V. (or its successor approved
 * by the membership of KDE e.V.), which shall act as a proxy
 * defined in Section 14 of version 3 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef INDEX_H
#define INDEX_H
#include <QObject>
#include <QTimer>
#include <Akonadi/Collection>
#include <akonadi/item.h>
#include "abstractindexer.h"

/**
 * Maintains the variuous indexers and databases
 */
class Index : public QObject
{
    Q_OBJECT
public:
    explicit Index(QObject* parent = 0);
    virtual ~Index();

    void removeDatabase();
    bool createIndexers();
    void index(const Akonadi::Item& item);
    void reindex(const Akonadi::Item& item);
    void move(const Akonadi::Item::List& items,
              const Akonadi::Collection& from,
              const Akonadi::Collection& to);
    void updateFlags(const Akonadi::Item::List& items,
                     const QSet<QByteArray>& addedFlags,
                     const QSet<QByteArray>& removed);
    void remove(const Akonadi::Item::List& items);
    void remove(const Akonadi::Collection& col);
    QList<AbstractIndexer*> indexersForCollection(const Akonadi::Collection& collection) const;
    qlonglong indexedItems(const qlonglong id);
    qlonglong indexedItemsInDatabase(const std::string& term, const QString& dbPath) const;
    void findIndexedInDatabase(QSet<Akonadi::Entity::Id>& indexed, Akonadi::Entity::Id collectionId, const QString& dbPath);
    void findIndexed(QSet<Akonadi::Item::Id>& indexed, Akonadi::Collection::Id);
    void scheduleCommit();

public Q_SLOTS:
    void commit();

private:
    void addIndexer(AbstractIndexer* indexer);
    AbstractIndexer* indexerForItem(const Akonadi::Item& item) const;

    QMap<QString, AbstractIndexer*> m_indexer;
    QTimer m_commitTimer;
};

#endif
